@@ -457,6 +457,20 @@ static const GDBusInterfaceVTable interface_vtable =
 	handle_method_call
 };
 
+static void
+cb_sms_delete_finish_retry(MMModemMessaging	*modemmessaging,
+			   GAsyncResult		*result,
+			   MMSms		*sms)
+{
+	g_autoptr(GError) error = NULL;
+
+	if(mm_modem_messaging_delete_finish(modemmessaging, result, &error)) {
+		DBG("Message delete finish");
+	} else {
+		DBG("Couldn't delete SMS - error: %s", error ? error->message : "unknown");
+	}
+}
+
 static void
 cb_sms_delete_finish(MMModemMessaging	*modemmessaging,
 		     GAsyncResult	*result,
@@ -467,7 +481,19 @@ cb_sms_delete_finish(MMModemMessaging	*modemmessaging,
 	if(mm_modem_messaging_delete_finish(modemmessaging, result, &error)) {
 		DBG("Message delete finish");
 	} else {
+		const char	*path;
+		path = mm_sms_get_path(sms);
 		DBG("Couldn't delete SMS - error: %s", error ? error->message : "unknown");
+		if(path) {
+			DBG("Trying to delete SMS again....");
+			mm_modem_messaging_delete(modem->modem_messaging,
+						  path,
+						  NULL,
+						  (GAsyncReadyCallback)cb_sms_delete_finish_retry,
+						  sms);
+		} else {
+			g_critical("mmsd-tng cannot process MMS at this time!");
+		}
 	}
 }
 
@@ -514,6 +540,7 @@ mmsd_process_sms(MMSms	*sms)
 			mms_service_push_notify(modem->service, data, data_len);
 			path = mm_sms_get_path(sms);
 			if(path) {
+				DBG("Deleting SMS Now...");
 				mm_modem_messaging_delete(modem->modem_messaging,
 							  path,
 							  NULL,
@@ -1258,8 +1285,8 @@ static int set_context(void)
 
 	DBG("Setting Context...");
 	if(modem->context_active) {
-		g_free(modem->context_interface);
-		g_free(modem->context_path);
+		g_clear_pointer(&modem->context_interface, g_free);
+		g_clear_pointer(&modem->context_path, g_free);
 	}
 	modem->context_active = FALSE;
 	interface_disconnected = FALSE;
@@ -1608,19 +1635,24 @@ static int modemmanager_init(void)
 static void modemmanager_exit(void)
 {
 	GDBusConnection	*connection = mms_dbus_get_connection ();
-	if(modem->plugin_registered == TRUE) {
-		mms_service_unregister(modem->service);
-		mms_service_unref(modem->service);
-		modem->modemsettings = NULL;
-	}
 	if(modem->modem_available) {
 		mmsd_mm_state(MMSD_MM_STATE_NO_MODEM);
 	}
 	if(modem->manager_available) {
 		mmsd_mm_state(MMSD_MM_STATE_NO_MANAGER);
 	}
+	if(modem->context_active) {
+		g_clear_pointer(&modem->context_interface, g_free);
+		g_clear_pointer(&modem->context_path, g_free);
+	}
+	if(modem->plugin_registered == TRUE) {
+		mms_service_unregister(modem->service);
+		modem->modemsettings = NULL;
+	}
+	mms_service_unref(modem->service);
 	g_dbus_connection_unregister_object(connection,
 					    modem->registration_id);
+	g_free(modem->registered_imsi);
 	g_free(modem->message_center);
 	g_free(modem->MMS_proxy);
 	g_free(modem->mms_apn);
